﻿Option Strict On
Option Explicit On
Imports System
Imports System.Collections
Imports System.ComponentModel

Namespace Microsoft.Samples.WindowsForms.Vb.Transparency


    <ProvideProperty("Transparency", GetType(Control))> _
    Public Class TransparencyHelper
        Inherits Component
        Implements System.ComponentModel.IExtenderProvider


#Region "Designer Init Code"
        Private components As System.ComponentModel.Container
        Private values As Hashtable
        Private activeControl As System.Windows.Forms.Control

        Public Sub New()
            '
            ' Required for Windows Form designer support.
            '
            InitializeComponent()
            values = New Hashtable()
        End Sub

        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                If Not components Is Nothing Then
                    components.Dispose()
                End If
            End If
            MyBase.Dispose(disposing)
        End Sub

        Private Sub InitializeComponent()
            Me.components = New System.ComponentModel.Container()
        End Sub

#End Region

        Const DEFAULT_VALUE As Decimal = 100
        Const DEFAULT_VALUE_STRING As String = "100.0" 'should be the same as DEFAULT_VALUE

        Function CanExtend(ByVal target As Object) As Boolean Implements IExtenderProvider.CanExtend
            'TODO: further restrict if TypeOf ... case to specific supported controls (Panel, UserControl, etc)
            If (TypeOf target Is Panel Or TypeOf target Is GroupBox Or TypeOf target Is UserControl) And Not TypeOf target Is TransparencyHelper Then
                Return True
            End If
            Return False
        End Function

        <DefaultValue(DEFAULT_VALUE_STRING)> _
        Public Function GetTransparency(ByVal ctrl As Control) As Decimal

            Dim myValue As Decimal = DEFAULT_VALUE

            If values.Contains(ctrl) Then
                myValue = CDec(values(ctrl))
            End If

            Return myValue
        End Function


        Public Sub SetTransparency(ByVal ctrl As Control, ByVal value As Decimal)

            doSetTransparency(ctrl, value)

            'only persist value to hash if not default
            If value <> DEFAULT_VALUE Then
                values(ctrl) = value
            End If

        End Sub

        Private Sub doSetTransparency(ByRef ctrl As Control, ByVal percentTransparent As Decimal)
            'validate inputs
            If percentTransparent < 0 Then percentTransparent = 0
            If percentTransparent > 100 Then percentTransparent = 100

            Dim alpha As Integer = CInt(percentTransparent / 100.0 * 255.0)

            ctrl.BackColor = Color.FromArgb(alpha, ctrl.BackColor)

        End Sub

        Public Function ShouldSerializeTransparency(ByVal ctrl As Control) As Boolean

            'only serialize value if supported by control
            If Not CanExtend(ctrl) Then
                Return False
            End If

            'only serialize value if not the default value
            If Me.GetTransparency(ctrl) <> DEFAULT_VALUE Then
                Return False
            Else
                Return True
            End If
        End Function
    End Class
End Namespace